package it.pkg.api.config;

import cn.hutool.json.JSONUtil;
import it.pkg.infra.data.model.CommonConstant;
import com.hikcreate.infra.framework.core.Result;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;


/**
 * @author Administrator
 */
@Slf4j
@Configuration
@Setter(onMethod = @__(@Autowired))
public class Interceptor implements HandlerInterceptor {

    private RedisTemplate<String, String> redisTemplate;
    private CommonConfig commonConfig;


    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        log.info("***bh.student.app.web.Interceptor.preHandle***");
        String token = request.getHeader(CommonConstant.TOKEN);
        String uri = request.getRequestURI();
        if (uri.contains(CommonConstant.SWAGGER) || uri.contains(CommonConstant.WEB_SERVER)) {
            return true;
        }
        Result<Void> responseResult;
        response.setHeader(CommonConstant.CONTENT_TYPE, CommonConstant.APPLICATION_JSON);
        //redis中保存了用户信息 需要的话可以反序列化取值
        if (StringUtils.isEmpty(token) || Boolean.FALSE.equals(redisTemplate.opsForHash().hasKey(commonConfig.userSessionPrefix + token, CommonConstant.TOKEN))) {
            responseResult = new Result<>();
            responseResult.setCode(CommonConstant.ERROR_CODE);
            responseResult.setMessage(CommonConstant.LOGIN_EXPIRED);
            PrintWriter pw = response.getWriter();
            pw.write(JSONUtil.toJsonStr(responseResult));
            pw.flush();
            pw.close();
            return false;
        }
        return true;
    }


    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object object, ModelAndView modelAndView) {
        //后处理回调方法，实现处理器的后处理（但在渲染视图之前），此时我们可以通过modelAndView（模型和视图对象）
        //对模型数据进行处理或对视图进行处理，modelAndView也可能为null
    }


    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object object, Exception exception) {
        //整个请求处理完毕回调方法，即在视图渲染完毕时回调，如性能监控中我们可以在此记录结束时间并输出消耗时间，还可以进行一些资源清理，
        //类似于try-catch-finally中的finally，但仅调用处理器执行链中preHandle返回true的拦截器的afterCompletion。
    }
}
